@@ -12,6 +12,7 @@
 
 #![cfg_attr(feature = "aggressive_lint", deny(warnings))]
 
+use std::fs::create_dir_all;
 use std::marker::Unpin;
 use std::path::Path;
 use std::pin::Pin;
@@ -33,6 +34,7 @@ mod bio {
     #[cfg(feature = "compression")]
     pub(super) use std::io::prelude::*;
     pub(super) use std::io::ErrorKind;
+    pub(super) use tempfile::NamedTempFile;
 }
 
 use crate::config::Config;
@@ -419,17 +421,6 @@ where
 {
     let (bucket, key, download_path) = (bucket.as_ref(), key.as_ref(), download_path.as_ref());
 
-    // If we're trying to download into a directory, assemble the path for the user
-    let file = if !download_path.is_dir() {
-        download_path.to_path_buf()
-    } else {
-        let s3filename = key
-            .split('/')
-            .next_back()
-            .ok_or(Error::CouldNotParseS3Filename)?;
-        download_path.join(s3filename)
-    };
-
     let stat = head_object_request(s3, bucket, key).await?;
     let total_size = stat
         .ok_or_else(|| Error::GetObjectInvalidKey(key.into()))?
@@ -465,24 +456,65 @@ where
         }
     }
 
+    let directory_path = if !download_path.is_file() && !download_path.is_dir() {
+        // If the specified destination directory doesn't already exist,
+        // see if the directory structure needs to be made
+        let directory_path = if !download_path.to_string_lossy().ends_with('/') {
+            let mut path = download_path.to_path_buf();
+            path.pop();
+            path
+        } else {
+            download_path.to_path_buf()
+        };
+
+        info!("Creating directory path {:?}", &directory_path);
+        create_dir_all(&directory_path)?;
+
+        directory_path
+    } else {
+        download_path
+            .parent()
+            .expect("must have a parent path")
+            .to_path_buf()
+    };
+
+    let tempfile = bio::NamedTempFile::new_in(directory_path)?;
+
     if decompress {
         #[cfg(feature = "compression")]
         {
             let file_output: Box<dyn bio::Write + Send + Sync + Unpin> =
-                Box::new(GzDecoder::new(bio::File::create(file)?));
+                Box::new(GzDecoder::new(bio::File::create(tempfile.path())?));
             let file_output = Arc::new(Mutex::new(file_output));
             let downloader = DownloadCompressed::new(file_output, bucket, key, total_size);
-            run_downloader(s3.clone(), downloader, decompress).await
+            run_downloader(s3.clone(), downloader, decompress).await?;
         }
         #[cfg(not(feature = "compression"))]
         {
             panic!("compression feature not enabled");
         }
     } else {
-        let file_output = bio::File::create(file)?;
-        let downloader = DownloadMultiple::new(file_output, bucket, key, total_size);
-        run_downloader(s3.clone(), downloader, decompress).await
+        let downloader =
+            DownloadMultiple::new(bio::File::create(tempfile.path())?, bucket, key, total_size);
+        run_downloader(s3.clone(), downloader, decompress).await?;
     }
+
+    // If we're trying to download into a directory, assemble the path for the user
+    let save_path = if !download_path.is_dir() {
+        download_path.to_path_buf()
+    } else {
+        let s3filename = key
+            .split('/')
+            .next_back()
+            .ok_or(Error::CouldNotParseS3Filename)?;
+        download_path.join(s3filename)
+    };
+
+    tempfile.persist(&save_path)?;
+
+    info!("File downloaded to {:?}", &save_path);
+
+    Ok(())
 }
 
 async fn download_streaming_range<T>(
@@ -525,6 +557,7 @@ pub(in crate) async fn download_with_dir<T>(
     s3_prefix: &str,
     s3_suffix: &str,
     local_dir: impl AsRef<Path>,
+    decompress: bool,
 ) -> Result<()>
 where
     T: S3 + Sync + Send + Clone,
@@ -538,7 +571,23 @@ where
     let key = format!("{}", Path::new(s3_prefix).join(s3_suffix).display());
     let dest_path = format!("{}", dest_path.display());
 
-    download(s3, bucket, &key, &dest_path).await?;
+    let file_is_compressed = dest_path.ends_with(".gz");
+
+    if decompress && file_is_compressed {
+        #[cfg(feature = "compression")]
+        {
+            let dest_path = dest_path
+                .strip_suffix(".gz")
+                .expect("Should have a gz suffix");
+            download_decompressed(s3, bucket, &key, dest_path).await?;
+        }
+        #[cfg(not(feature = "compression"))]
+        {
+            panic!("compression feature not enabled");
+        }
+    } else {
+        download(s3, bucket, &key, &dest_path).await?;
+    }
 
     Ok(())
 }
